@@ -1,30 +1,57 @@
 from django.db.models import QuerySet
 from django.views.decorators.csrf import csrf_exempt
+from django.utils.decorators import method_decorator
 from django.urls import path
 from django.http import HttpRequest
+from django.core.signing import BadSignature, SignatureExpired
+from django.contrib.auth import get_user_model
+from django.core.exceptions import ObjectDoesNotExist
 
 from typing import List
 from http import HTTPStatus
 
 from django_routeview import RouteView, urlpatterns
 
-from .responses import APIResponse, QuerySuccessful, CreationSuccessful, NotFound, NotAllowed, Conflict
+from .responses import APIResponse, QuerySuccessful, CreationSuccessful, NotFound, NotAllowed, Conflict, InvalidToken
 from .JSONMixin import JSONMixin
+from .Token import Token
+from .decorators import catch_exceptions
 
 
 class APIView(RouteView):
     """
      Auto registered view on self.route path
      Describe the endpoints associated with a model
+
+
+     `model:JSONMixin|django.db.Model`
+
+     `queryset:django.db.QuerySet:optional` # Default to `model.objects.all()`
+
+     `singular_name:str:optional` # Default to `model._meta.verbose_name or model.__name__`
+
+     `plural_name:str:optional` # Default to `model._meta.verbose_name_plural or f"{model.__name__}s"`
+
+     `enforce_authentification:bool:optional` Default to `False`
+
+     `http_method_names:list[str]:optional` Default to `["get", "post", "put", "patch", "delete", "head", "options"]`
     """
 
     model:JSONMixin = None
     queryset:QuerySet = None
     singular_name:str = None
     plural_name:str = None
-    authentification:bool = False
+    enforce_authentification:bool = False
     http_method_names:List[str] = ["get", "post", "put", "patch", "delete", "head", "options"]
 
+    _permissions_match_table = {
+        'GET': "view",
+        'PATCH': "change",
+        'POST': "add",
+        'PUT': "add",
+        'DELETE': "delete"
+    }
+
     def _add_route_(self) -> None:
         if self.route is not None:
             if self.name is None:
@@ -40,6 +67,9 @@ class APIView(RouteView):
                 urlpatterns.append(path(f"{self.route}<int:id>", self.as_view(), name=self.name))
 
     def __init__(self, *args, **kwargs):
+        if self.model is None:
+            raise ValueError(f"APIView {self.__name__} requires a model")
+
         super().__init__(*args, **kwargs)
 
         if self.queryset is None:
@@ -51,8 +81,33 @@ class APIView(RouteView):
         if self.plural_name is None:
             self.plural_name = self.model._meta.verbose_name_plural or f"{self.model.__name__}s"
 
+    @catch_exceptions
     @csrf_exempt
-    def dispatch(self, *args, **kwargs) -> APIResponse:
+    def dispatch(self, request:HttpRequest, *args, **kwargs) -> APIResponse:
+        headers = dict(request.headers)
+
+        if not self.enforce_authentification:
+            return super().dispatch(request, *args, **kwargs)
+
+        if not 'Authorization' in headers:
+            return InvalidToken("Authentification required")
+
+        token = Token(signed_data=headers['Authorization'].split("")[1])
+        try:
+            token.unsign()
+        except BadSignature:
+            return InvalidToken("Invalid signature")
+        except SignatureExpired:
+            return InvalidToken("Token expired")
+
+        try:
+            user = get_user_model().objects.get(id=token.uid)
+        except (KeyError, ObjectDoesNotExist):
+            return InvalidToken("Invalid body")
+
+        if not user.has_perm(f'api.{self.match_table[request.method]}_{self.verbose_name}') and not user.id == request.path_info.split('/')[-1].split('?')[0]:
+            return NotAllowed()
+
         return super().dispatch(*args, **kwargs)
 
     def get(self, request:HttpRequest, id:int=None, *args, **kwargs) -> APIResponse:
